/**
 * BH1750_LightSensor
 * Klasse fuer den Zugriff auf einen Lichtsensor mit I2C.
 *
 * @mc       Arduino/ESP8266
 * @autor    Harald Sattler
 * @version  1.0
 * @created  17.01.2019
 *
 */

#ifndef BH1750_LightSensor_H
#define BH1750_LightSensor_H

#include "Arduino.h"
#include "Configuration.h"

class BH1750 {
public:
	BH1750(byte adr, boolean isInverted, uint8_t constrainMin, uint8_t constrainMax, boolean autoScale = true, uint16_t ldrHysteresis = 50, uint16_t minLdr = 0, uint16_t maxLdr = 1023);

	byte value();

private:
	byte _adr;
	boolean _isInverted;
	boolean _autoScale;
	uint16_t _lastValue;
	byte _outputValue;
	uint16_t _min;
	uint16_t _max;
	uint16_t _ldrMin;
	uint16_t _ldrMax;
	uint16_t _ldrHysteresis;
	uint8_t _constrainMin;
	uint8_t _constrainMax;
	unsigned long _measurementStarted;	// Zeit nach Start der Messung muss abgewartet werden
	boolean _messungLaeuft;
	byte _buff[2];
	
	int BH1750_Read(int address);		//

};

#endif
